package com.sea.gbgs5ktest;

import com.cinterion.io.ATCommandFailedException;
import java.io.*;
import com.sea.log.Logger;
import com.sea.modem.*;
import com.sea.slave.SlaveListener;
import com.sea.utils.*;
import java.util.Enumeration;
import java.util.Timer;
import java.util.Vector;

public class Control extends Thread implements SlaveListener {

// --------------------------------------------------
// Public Properties
// --------------------------------------------------
// --------------------------------------------------
// Private Constants
// --------------------------------------------------
// Machine states
private static final int CTRL_INIT = 0;
private static final int CTRL_CREATE = 10;
private static final int CTRL_START = 20;
private static final int CTRL_READY = 1000;
private static final int CTRL_STOP = 2000;
private static final int CTRL_FINISHED = 9999;

// How long sleep when ready [ms]
private static final int READY_DELAY = 20000;

// --------------------------------------------------
// Private Properties
// --------------------------------------------------
private static Control instance = null;
private static boolean bRunning = false;

// Debug Level
public static int iLogLevel = Logger.INFO;

// Machine state
private static int iMachine;
private static boolean bWantedToStop;
private static Vector vDeviceInfo;

// Controlled threads
//private static A4A3NGMultiplexer mux0;
private static Console consl;
private static ConsoleReporter conslrep;
private static Slave slave;
private static Watchdog wdg;

// Managed objects
//private static DInManager dinmgr;
//private static DOutManager doutmgr;
// Controlled timer and its tasks
private Timer tmr1;
private Info ttInfo;
//private UserLogger ttUserLogger;
//private EventScheduler ttEventScheduler;

// Controlled AT channels
private ATSender ata, atc;
private ATSender atb;

// --------------------------------------------------
// Class Code
// --------------------------------------------------
/**
 * Constructor
 */
public Control() {
  ata = null;
  //atb = null;
  atc = null;
}

/**
 * getInstance()
 *
 * @return
 */
public static Control getInstance() {
  if (instance == null) {
    instance = new Control();
  }
  return instance;
}

/**
 * startRunning
 */
public void startRunning() {
  bWantedToStop = false;
  bRunning = true;
  this.start();
}

/**
 * stopRunning
 */
public void stopRunning() {
  if (!bWantedToStop) {
    bWantedToStop = true;
    instance.interrupt();
  }
}

/**
 * Debug printing
 */
protected static void log(String line) {
  log(line, Logger.INFO);
}

/**
 * Debug printing
 */
protected static void log(String line, int level) {
  if (iLogLevel >= level) {
    Logger.log("@Control", line);
  }
}

/**
 * run
 */
public void run() {

  machineJumpTo(CTRL_INIT);
  while (bRunning) {
    try {
      switch (iMachine) {
        case CTRL_INIT:
          ctrlMachineInit();
          break;
        case CTRL_CREATE:
          ctrlMachineCreate();
          break;
        case CTRL_START:
          ctrlMachineStart();
          break;
        case CTRL_READY:
          ctrlMachineReady();
          break;
        case CTRL_STOP:
          ctrlMachineStop();
          break;
        case CTRL_FINISHED:
          ctrlMachineFinished();
          break;
        default:

          // inpossible state
          machineJumpTo(CTRL_INIT);
      }
    } catch (Exception e) {
      // An exception occured. Go the interrupted step again.
      log("[EXCEPTION] - " + e.toString(), Logger.ERROR);
      e.printStackTrace();
      try {
        sleep(500);
      } catch (Exception ignored) {
      }
    }

    if (bWantedToStop && bRunning) {
      machineJumpTo(CTRL_STOP);
    }

  }

  log("Thread finished.", Logger.ALWAYS);
}

/**
 * ctrlMachineInit
 */
private void ctrlMachineInit() {

  // Create AT channels
  try {
    if (ata == null) {
      ata = new ATSender(false, "ata");
    }
    if (atb == null) {
      atb = new ATSender(true, "atb");
    }
    if (atc == null) {
      atc = new ATSender(false, "atc");
    }
  } catch (ATCommandFailedException ex) {
    log("[ERROR] - Cannot create AT channels", Logger.ERROR);
    // ```TODO Exceptions from AtCommand constructor have to cause RESET of the device
  }
  selectStdout(ata, "usb");

  //Cfg.init();
  //Cfg.detectSerialNumber();
  vDeviceInfo = prepareDeviceInfo();
  printDeviceInfo(vDeviceInfo);

  log("Comm ports available='" + System.getProperty("microedition.commports") + "'");
  log("Thread started.", Logger.ALWAYS);

  machineJumpTo(CTRL_CREATE);
}

/**
 * ctrlMachineStart
 */
private void ctrlMachineCreate() throws ATCommandFailedException, IOException {

  log("Creating Console");
  consl = Console.getInstance();
  consl.registerAtCommand(atb);
  consl.setDeviceInfo(vDeviceInfo);
  conslrep = ConsoleReporter.getInstance();
  conslrep.registerAtCommand(atc);
  log("Creating Slave");
  slave = Slave.getInstance();
  slave.addListener(this);

  log("Creating Watchdog");
  wdg = Watchdog.getInstance();
  wdg.registerAtSender(ata);

  // Timer based tasks 
  tmr1 = new Timer();
  ttInfo = new Info();
  //ttUserLogger = UserLogger.getInstance();
  //ttUserLogger.registerHttpLogger(reporterHttp);

  machineJumpTo(CTRL_START);
}

/**
 * ctrlMachineStart
 */
private void ctrlMachineStart() {
  log("Starting threads.");

  //evproc.startRunning();
  //modem.startRunning();
  //atf.startRunning();
  consl.startRunning();
  conslrep.startRunning();
  slave.startRunning();
  wdg.startRunning();

  tmr1.scheduleAtFixedRate(ttInfo, 0, 10000);
  //tmr1.scheduleAtFixedRate(ttUserLogger, 0, 1000);
  /*
   // Example of TimerTask
   tmr1.schedule(new TimerTask() {
   public void run() {
   }
   }, 10000);
   */

  machineJumpTo(CTRL_READY);
}

/**
 * ctrlMachineReady
 */
private void ctrlMachineReady() throws InterruptedException {

  log(getStatus(0), Logger.ALWAYS);

  // Watch all other threads and restart if any of them stops working
  //if (!wdg.isAlive() || !atf.isAlive()) {
  if (!wdg.isAlive() || !slave.isAlive()) {
    Watchdog.deviceReset();
  }

  delay(READY_DELAY);
}

/**
 * ctrlMachineStop
 */
private void ctrlMachineStop() {

  Vector threads = new Vector(); // Vector of Threads

  // --------------------------------------------------------
  // Stop of thread Info
  log("- Timer1(Info)");
  if (tmr1 != null) {
    tmr1.cancel();
  }

  // --------------------------------------------------------
  //log("- AIn, AOut, DIn, DOut");
  //elmgr = null;
  //aoutmgr = null;
  // --------------------------------------------------------
  log("Stopping threads:");
  try {
    if (conslrep != null) {
      log("- ConsoleReporter");
      conslrep.stopRunning();
      threads.addElement(conslrep);
    }
  } catch (Exception ignored) {
  }
  try {
    if (consl != null) {
      log("- Console");
      consl.stopRunning();
      threads.addElement(consl);
    }
  } catch (Exception ignored) {
  }
  try {
    if (wdg != null) {
      log("- Watchdog");
      wdg.stopRunning();
      threads.addElement(wdg);
    }
  } catch (Exception ignored) {
  }

  /*
    try {
    if (atf != null) {
      log("- ATFilter");
      atf.stopRunning();
      threads.addElement(atf);
    }
  } catch (Exception ignored) {
  }
   */
  try {
    if (slave != null) {
      log("- Slave");
      slave.stopRunning();
      threads.addElement(slave);
    }
  } catch (Exception ignored) {
  }

  // --------------------------------------------------------
  log("Waiting for all threads to die ...");

  boolean bAllIsTerminated = false;
  long lStartTime = Timed.currentTimeMillis();
  while (!bAllIsTerminated && !Timed.ist(lStartTime, 10000)) {
    Thread t;
    boolean bSomethingLives = false;
    for (Enumeration e = threads.elements(); e.hasMoreElements();) {
      t = (Thread) e.nextElement();
      if (t.isAlive()) {
        bSomethingLives = true;
        log(t.getClass() + " is ALIVE");
      } else {
        //log(t.getClass() + " is dead");
      }
    }

    if (bSomethingLives) {
      log("  wait ...");
      try {
        sleep(300);
      } catch (InterruptedException ex) {
      }
    } else {
      bAllIsTerminated = true;
    }

  }

  // --------------------------------------------------------
  log("Releasing AT Channels");
  log("- ATB");
  try {
    atb.release();
  } catch (Exception ex) {
  }

  log("- ATC");
  try {
    atc.release();
  } catch (Exception ex) {
  }

  log("- ATA1");
  selectStdout(ata, "USB1");  // Switch stdout to safe direction
  log("- ATA2");
  try {
    ata.release();
  } catch (Exception ex) {
  }

  log("Stop completed.");
  bRunning = false;

  machineJumpTo(CTRL_FINISHED);
}

/**
 * ctrlMachineFinished
 */
private void ctrlMachineFinished() {
  bRunning = false;
}

/**
 * delay
 *
 * @param millis long
 */
private void delay(long millis) {
  try {
    sleep(millis);
  } catch (InterruptedException ignored) {
  }
}

// Jump to named state of machine (dtMachineStateH)
private void machineJumpTo(int aNewState) {
  iMachine = aNewState;
}

/**
 * printDeviceInfo()
 *
 * @param src
 */
public static void printDeviceInfo(Vector src) {

  for (int i = 0; i < src.size(); i++) {
    log((String) src.elementAt(i), Logger.ALWAYS);
  }

}

/**
 * prepareDeviceInfo()
 *
 * @return
 */
public static Vector prepareDeviceInfo() {

  Vector result = new Vector(7);
  result.addElement("--- START --------------------------------------------------------");
  result.addElement("Product:       " + Main.PRODUCT_NAME + " (" + Main.PRODUCT_NUMBER + ")");
  result.addElement("Version:       " + Main.PRODUCT_VERSION);
  result.addElement("Manufacturer:  SEA s.r.o., Czech Republic");
  result.addElement("               www.seapraha.cz");
  result.addElement("Serial Number: " + formatSerialNumber());
  result.addElement("------------------------------------------------------------------");
  return result;
}

/**
 * printSerialNumber()
 *
 * @return String
 */
private static String formatSerialNumber() {

  return "<no s/n/>";
  /*
  if (Cfg.SERIAL_NUMBER.length() > 0) {
    return Cfg.SERIAL_NUMBER;
  } else {
    return "-";
  }
   */

}

/**
 * getStatus
 *
 * @return String
 */
private String getStatus(int index) {

  String result = "tick";
  return result;
}

/**
 * Set STDOUT
 *
 * @param destination "ASC0", "ASC1" or "NULL", "USB1"
 */
private void selectStdout(ATSender atc, String destination) {

  try {
    atc.send("at^scfg=\"Userware/Stdout\",\"" + destination + "\"\r");
  } catch (Exception ignored) {
  }

}

public void onButtonPressed(int counter) {
  log("onButtonPressed() - counter=" + counter);
}

public void onButtonDuringPowerUp() {
  log("onButtonDuringPowerUp()");
}

public void onDOutChangedByButton(int index, boolean value) {
  log("onDOutChangedByButton() - index=" + index + " value=" + value);
}

public void onDInChanged(int index, boolean value) {
  log("onDInChanged() - index=" + index + " value=" + value);
}

}
