package com.sea.slave;

import com.sea.utils.Crc8;
import com.sea.utils.Utils;
import java.io.IOException;

/**
 *
 * @author Vorisek
 */
class InputPacket {

// --------------------------------------------------
// Public Constants
// --------------------------------------------------
// --------------------------------------------------
// Public Properties
// --------------------------------------------------
// --------------------------------------------------
// Private Constants
// --------------------------------------------------
private static final int RESPONSE = 0x80;
// --------------------------------------------------
// Private Properties
// --------------------------------------------------
private final byte[] packetRaw = new byte[256];
private final byte[] packet = new byte[128];
private int remaining;
private int pos;

// --------------------------------------------------
// Methods Service
// --------------------------------------------------
public InputPacket() {
  pos = 0;
  remaining = 0;
}

// --------------------------------------------------
// Methods Public
// --------------------------------------------------
void parse(Response response, int expectedCmd, int minimalLen) throws IOException {

  // Decode HEX string from response into packetRaw
  // "{a+01234567}" -> {0x01, 0x23, 0x45, 0x67}
  byte bh, bl;
  int idxSrc = 3;
  int idxRaw = 0;
  while (idxSrc < response.len - 1) {
    bh = response.valueArr[idxSrc++];
    bl = response.valueArr[idxSrc++];
    packetRaw[idxRaw++] = hexNibbles2Byte(bh, bl);
  }
  int lenRaw = idxRaw;
  //System.out.println("Decode: '" + Utils.pckt2str(packetRaw, 0, lenRaw)); // trace

  // Process decoded data
  if (lenRaw < (minimalLen + 3)) {
    throw new ArrayIndexOutOfBoundsException("Incoming packet too short (needed " + (minimalLen + 3) + " bytes)");
  }

  int len = packetRaw[0] & 0x000000FF;
  int crc = packetRaw[1] & 0x000000FF;
  int cmd = packetRaw[2] & 0x000000FF;
  int crcCalc = Crc8.countCrc8(packetRaw, 2, len);
  if (crcCalc != crc) {
    throw new IOException("CRC doesn't match: expected=0x" + Utils.intToHex(crcCalc, 2) + " read=0x" + Utils.intToHex(crc, 2));
  }

  if (cmd != (expectedCmd | RESPONSE)) {
    throw new IOException("CMD doesn't match: expected=0x" + Utils.intToHex(expectedCmd, 2) + " read=0x" + Utils.intToHex(cmd, 2));
  }

  System.arraycopy(packetRaw, 3, packet, 0, len - 1); // Copy only DATA (no CMD)
  remaining = len - 1; // strip CMD
  pos = 0;
  //System.out.println(Utils.byteArrayToHexStringWithSpaces(packet, 0, remaining));

}

int readU8() throws IOException {
  if (remaining < 1) {
    throw new IOException("Not enough bytes");
  }
  int b = packet[pos++];
  remaining--;
  return b & 0x000000FF;
}

int readU16() throws IOException {
  if (remaining < 2) {
    throw new IOException("Not enough bytes");
  }

  int result = 0;
  result |= (packet[pos++] << 8) & 0x0000FF00;
  result |= (packet[pos++]) & 0x000000FF;
  remaining -= 2;
  return result;
}

int readU32() throws IOException {
  if (remaining < 4) {
    throw new IOException("Not enough bytes");
  }

  int result = 0;
  result |= (packet[pos++] << 24) & 0xFF000000;
  result |= (packet[pos++] << 16) & 0x00FF0000;
  result |= (packet[pos++] << 8) & 0x0000FF00;
  result |= (packet[pos++]) & 0x000000FF;
  remaining -= 4;
  return result;
}

private byte hexNibbles2Byte(byte bh, byte bl) {
  int bhx = 0;
  if ((bh >= '0') && (bh <= '9')) {
    bhx = bh - '0';
  } else if ((bh >= 'A') && (bh <= 'F')) {
    bhx = bh - 55; // - 'A' + 10;
  }
  int blx = 0;
  if ((bl >= '0') && (bl <= '9')) {
    blx = bl - '0';
  } else if ((bl >= 'A') && (bl <= 'F')) {
    blx = bl - 55; // - 'A' + 10;
  }
  return (byte) ((bhx << 4) | blx);
}

}
