package com.sea.slave;

import com.sea.log.Logger;
import com.sea.utils.Utils;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * 14:53:54.763: - at^sspi=0000,0000,0000
 * 14:53:54.775: + at^sspi=0000,0000,0000
 * 14:53:55.308: + CONNECT
 * 14:54:00.971: - <a5801D703>
 * 14:54:01.004: + {a+}
 * 14:54:03.773: - <b590010>
 * 14:54:03.797: + {b+0B29830100C588D70103000000AAAAAAAAAA}
 * 14:54:07.956: - #
 * 14:54:08.283: + OK
 *
 * @author Vorisek
 */
public class Protocol {

// --------------------------------------------------
// Public Constants
// --------------------------------------------------
// --------------------------------------------------
// Public Properties
// --------------------------------------------------
// CMD_INFO
public int productNumber = -1;
public int productVersion = -1;
public int hwVariant = -1;
public int flags = -1;
// CMD_IO
public int doutResp = -1;
public int din = -1;
public int adVgsm = -1;
public int adVin = -1;
public int adVbus = -1;
public int adTemp = -1;
public int btnCnt = -1; // -1=never communication, 0=never pressed, 1, 2, ... 254, 255, 1, 2, ...
public long upTime = -1;

// --------------------------------------------------
// Private Constants
// --------------------------------------------------
private final static int I2C_ADDRESS = 0x58;

private final static int MAX_PACKET_LEN = 256;  // 256 should be enough, but just to be sure

// Commands
protected static final int CMD_INFO = 0x03;
protected static final int CMD_IO = 0x10;
protected static final int CMD_CONFIG = 0x17;
protected static final int CMD_RESTART = 0x31;

private static final int VIN_CORRECTION = 1500; // [mV] Correction for diodes in the power path

// --------------------------------------------------
// Private Properties
// --------------------------------------------------
public int iLogLevel = Logger.INFO;
protected InputStream in = null;
protected OutputStream out = null;

private final OutputPacket packeto = new OutputPacket();
private final InputPacket packeti = new InputPacket();
protected byte[] packetIn = new byte[MAX_PACKET_LEN];
protected byte[] tempPreReadBuffer;

protected int addressWrite;
protected int addressRead;
private StringHexComposer shc = new StringHexComposer(256);

// --------------------------------------------------
// Methods Service
// --------------------------------------------------
/**
 * Constructor
 *
 * @param sin
 * @param sout
 */
public Protocol(InputStream sin, OutputStream sout) {

  in = sin;
  out = sout;
  setAddress(I2C_ADDRESS);

  tempPreReadBuffer = new byte[2];
  tempPreReadBuffer[0] = 0x00;
  tempPreReadBuffer[1] = 0x14;

}

// --------------------------------------------------
// Methods Public
// --------------------------------------------------
/**
 * Read slave information
 *
 * @throws java.lang.Exception
 */
public void info() throws Exception {

  // master->slave:	CMD_INFO1
  // slave->master:	CMD_INFO_RESP1 PROTO_VERSION1 PRODUCT_NUMBER4 PRODUCT_VERSION4 HW_VARIANT1
  int len;
  byte[] buf;
  Response response;

  packeto.begin(CMD_INFO);
  packeto.end();
  buf = packeto.data();
  len = packeto.size();

  transact(addressWrite, buf, 0, len);

  len = tempPreReadBuffer.length;
  buf = tempPreReadBuffer;

  response = transact(addressRead, buf, 0, len);

  packeti.parse(response, CMD_INFO, 10);
  productNumber = packeti.readU32();
  productVersion = packeti.readU32();
  hwVariant = packeti.readU8();
  flags = packeti.readU8();

}

/**
 * Read slave information
 *
 * @throws java.lang.Exception
 */
public void io(int dout, int doutPulses, int ledState, int ledGsm, int ledOut1, int ledOut2) throws Exception {

  // master->slave:	CMD_IO1 DOUT1 DOUT_PULSES1 [LED_STATE1 LED_GSM1 LED_OUT11 LED_OUT21]
  // slave->master:	CMD_IO_RESP1 DOUT1 DIN1 ADVGSM2 ADVIN2 ADVBUS2 TEMP2 BTNCNT1  UPTIME4
  int len;
  byte[] buf;
  Response response;

  packeto.begin(CMD_IO);
  packeto.writeU8(dout);
  packeto.writeU8(doutPulses);
  packeto.writeU8(ledState);
  packeto.writeU8(ledGsm);
  packeto.writeU8(ledOut1);
  packeto.writeU8(ledOut2);
  packeto.end();
  buf = packeto.data();
  len = packeto.size();

  transact(addressWrite, buf, 0, len);

  len = tempPreReadBuffer.length;
  buf = tempPreReadBuffer;

  response = transact(addressRead, buf, 0, len);

  packeti.parse(response, CMD_IO, 16);
  this.doutResp = packeti.readU8();
  this.din = packeti.readU8();
  this.adVgsm = packeti.readU16();
  this.adVin = packeti.readU16() + VIN_CORRECTION;
  this.adVbus = packeti.readU16();
  this.adTemp = packeti.readU16();
  this.btnCnt = packeti.readU8();
  this.upTime = packeti.readU32();

}

/**
 * Read slave information
 *
 * @throws java.lang.Exception
 */
public void io(int dout, int doutPulses) throws Exception {

  // master->slave:	CMD_IO1 DOUT1 DOUT_PULSES1 [LED_STATE1 LED_GSM1 LED_OUT11 LED_OUT21]
  // slave->master:	CMD_IO_RESP1 DOUT1 DIN1 ADVGSM2 ADVIN2 ADVBUS2 TEMP2 BTNCNT1  UPTIME4
  int len;
  byte[] buf;
  Response response;

  packeto.begin(CMD_IO);
  packeto.writeU8(dout);
  packeto.writeU8(doutPulses);
  packeto.end();
  buf = packeto.data();
  len = packeto.size();

  transact(addressWrite, buf, 0, len);

  len = tempPreReadBuffer.length;
  buf = tempPreReadBuffer;

  response = transact(addressRead, buf, 0, len);

  packeti.parse(response, CMD_IO, 16);
  this.doutResp = packeti.readU8();
  this.din = packeti.readU8();
  this.adVgsm = packeti.readU16();
  this.adVin = packeti.readU16() + VIN_CORRECTION;
  this.adVbus = packeti.readU16();
  this.adTemp = packeti.readU16();
  this.btnCnt = packeti.readU8();
  this.upTime = packeti.readU32();

}

/**
 * Set slave processor configuration
 * 
 * @param baudrate int, 115200 = 115200 baud, 19200 = 19200 baud
 * @param bits int, bits per byte. For example 8E1 means 1 START + 8 DATA + 1 PARITY + 1 STOP = 11, 8N1 means 1 START + 8 DATA + 1 STOP = 10
 * @throws java.lang.Exception
 */
public void config(int baudrate, int bits) throws Exception {

  // master->slave:	CMD_CONFIG(1) [COMM_BAUDRATE(2) COMM_BITS(1)]
  // slave->master:	CMD_IO_RESP(1) COMM_BAUDRATE(2) COMM_BITS(1)
  int len;
  byte[] buf;
  Response response;
  
  if ((bits < 9) || (bits > 11))
    throw new IllegalArgumentException("'bits' parameter have to be between 9 and 11.");

  packeto.begin(CMD_CONFIG);
  packeto.writeU16(baudrate / 10); // COMM_BAUDRATE
  packeto.writeU8(bits); // COMM_BITS
  packeto.end();
  buf = packeto.data();
  len = packeto.size();

  transact(addressWrite, buf, 0, len);

  len = tempPreReadBuffer.length;
  buf = tempPreReadBuffer;

  response = transact(addressRead, buf, 0, len);

  packeti.parse(response, CMD_CONFIG, 3);

}

/**
 * Restart
 *
 * @throws java.lang.Exception
 */
public void restart() throws Exception {

  // master->slave:	CMD_SHUTDOWN1 MODE1 DELAY2
  // slave->master:	CMR_SHUTDOWN_RESP1 MODE1 DELAY2
  int len;
  byte[] buf;
  Response response;

  packeto.begin(CMD_RESTART);
  packeto.writeU8(0x01); // MODE
  packeto.writeU16(0x0001); // DELAY
  packeto.end();
  buf = packeto.data();
  len = packeto.size();

  transact(addressWrite, buf, 0, len);

  len = tempPreReadBuffer.length;
  buf = tempPreReadBuffer;

  response = transact(addressRead, buf, 0, len);

  packeti.parse(response, CMD_RESTART, 3);

}

/**
 * Go to sleep mode (=GSM modem is powered off until L->H on main power occurs) or durationMinutes is elapsed
 *
 * @throws java.lang.Exception
 * @param durationMinutes How long to sleep [min], 0xFFFF = forever
 */
public void sleep(int durationMinutes) throws IOException, Exception {
  
  int len;
  byte[] buf;
  Response response;

  packeto.begin(CMD_RESTART);
  packeto.writeU8(0x03); // MODE
  packeto.writeU16(durationMinutes); // DELAY
  packeto.end();
  buf = packeto.data();
  len = packeto.size();

  transact(addressWrite, buf, 0, len);

  len = tempPreReadBuffer.length;
  buf = tempPreReadBuffer;

  response = transact(addressRead, buf, 0, len);

  packeti.parse(response, CMD_RESTART, 4);

}

// --------------------------------------------------
// Methods Private
// --------------------------------------------------
/**
 * Make one I2C transaction WRITE/RECEIVE. Blocking method.
 *
 * @param address
 * @param pcktOut
 * @param offset
 * @param length
 * @return
 * @throws java.lang.Exception
 */
protected Response transact(int address, byte[] pcktOut, int offset, int length) throws Exception {

  int readBytes;
  Response result = new Response();

  // Send query: <a00112233>
  shc.reset();
  shc.append("<a").appendU8(address).appendHexString(pcktOut, offset, length).append('>');
  shc.writeToStream(out);
  //System.out.println("Write: '" + shc.toString() + "'");
  out.flush();

  // Receive response: {a+00112233}
  int firstFree = 0;
  do {
    if (firstFree == packetIn.length) {
      throw new Exception("I2C response too long: " + Utils.pckt2str(packetIn, 0, firstFree));
    }
    readBytes = in.read(packetIn, firstFree, packetIn.length - firstFree);
    firstFree += readBytes;
  } while (packetIn[firstFree - 1] != '}');

  if ((firstFree < 4) || (packetIn[2] != '+')) {
    throw new Exception("Invalid response to I2C transact(): " + Utils.pckt2str(packetIn, 0, firstFree));
  }
  //System.out.println("Read: '" + Utils.pckt2str(packetIn, 0, firstFree)); // trace
  result.valueArr = packetIn; 
  result.len = firstFree;

  return result;

}

protected final void setAddress(int address) {

  addressWrite = address & ~0x01;
  addressRead = address | 0x01;

}

}
